/*****************************************************************************

  FileName:		Q850.c

  Contents:		ITU-T Q.850 cause codes

  License/Copyright:

  Copyright (c) 2008, Stefan Knoblich, axsentis GmbH. All rights reserved.
  email: s.knoblich@axsentis.de

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of the axsentis GmbH nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/
#include <stdlib.h>

#include "Q931.h"

struct Q850CauseName
{
	const int id;
	const char *name;

} Q850CauseNames[] = {
	{ Q850_CAUSE_NONE, "None" },
	{ Q850_CAUSE_UNALLOCATED, "Unallocated" },
	{ Q850_CAUSE_NO_ROUTE_TRANSIT_NET, "No route transit net" },
	{ Q850_CAUSE_NO_ROUTE_DESTINATION, "No route destination" },
	{ Q850_CAUSE_CHANNEL_UNACCEPTABLE, "Channel unacceptable" },
	{ Q850_CAUSE_CALL_AWARDED_DELIVERED, "" },
	{ Q850_CAUSE_NORMAL_CLEARING, "Normal clearing" },
	{ Q850_CAUSE_USER_BUSY, "User busy" },
	{ Q850_CAUSE_NO_USER_RESPONSE, "No user response" },
	{ Q850_CAUSE_NO_ANSWER, "No answer" },
	{ Q850_CAUSE_SUBSCRIBER_ABSENT, "Subscriber absent" },
	{ Q850_CAUSE_CALL_REJECTED, "Call rejected" },
	{ Q850_CAUSE_NUMBER_CHANGED, "Number changed" },
	{ Q850_CAUSE_REDIRECTION_TO_NEW_DESTINATION, "Redirection to new destination" },
	{ Q850_CAUSE_EXCHANGE_ROUTING_ERROR, "Exchange routing error" },
	{ Q850_CAUSE_DESTINATION_OUT_OF_ORDER, "Destination out of order" },
	{ Q850_CAUSE_INVALID_NUMBER_FORMAT, "Invalid number format" },
	{ Q850_CAUSE_FACILITY_REJECTED, "Facility rejected" },
	{ Q850_CAUSE_RESPONSE_TO_STATUS_ENQUIRY, "Response to STATUS ENQUIRY" },
	{ Q850_CAUSE_NORMAL_UNSPECIFIED, "Unspecified" },
	{ Q850_CAUSE_NORMAL_CIRCUIT_CONGESTION, "Circuit congestion" },
	{ Q850_CAUSE_NETWORK_OUT_OF_ORDER, "Network out of order" },
	{ Q850_CAUSE_NORMAL_TEMPORARY_FAILURE, "Temporary failure" },
	{ Q850_CAUSE_SWITCH_CONGESTION, "Switch congestion" },
	{ Q850_CAUSE_ACCESS_INFO_DISCARDED, "Access info discarded" },
	{ Q850_CAUSE_REQUESTED_CHAN_UNAVAIL, "Requested channel unavailable" },
	{ Q850_CAUSE_PRE_EMPTED, "Pre-empted" },
	{ Q850_CAUSE_FACILITY_NOT_SUBSCRIBED, "Facility not subscribed" },
	{ Q850_CAUSE_OUTGOING_CALL_BARRED, "Outgoing call barred" },
	{ Q850_CAUSE_INCOMING_CALL_BARRED, "Incoming call barred" },
	{ Q850_CAUSE_BEARERCAPABILITY_NOTAUTH, "" },
	{ Q850_CAUSE_BEARERCAPABILITY_NOTAVAIL, "Bearer capability not available" },
	{ Q850_CAUSE_SERVICE_UNAVAILABLE, "Service unavailable" },
	{ Q850_CAUSE_BEARERCAPABILITY_NOTIMPL, "Bearer capability not implemented" },
	{ Q850_CAUSE_CHAN_NOT_IMPLEMENTED, "Channel not implemented" },
	{ Q850_CAUSE_FACILITY_NOT_IMPLEMENTED, "Facility not implemented" },
	{ Q850_CAUSE_SERVICE_NOT_IMPLEMENTED, "Service not implemented" },
	{ Q850_CAUSE_INVALID_CALL_REFERENCE, "Invalid call reference" },
	{ Q850_CAUSE_INCOMPATIBLE_DESTINATION, "Incompatible destination" },
	{ Q850_CAUSE_INVALID_MSG_UNSPECIFIED, "Message, unspecified" },
	{ Q850_CAUSE_MANDATORY_IE_MISSING, "Mandatory IE missing" },
	{ Q850_CAUSE_MESSAGE_TYPE_NONEXIST, "Message type nonexistant" },
	{ Q850_CAUSE_WRONG_MESSAGE, "Wrong message" },
	{ Q850_CAUSE_IE_NONEXIST, "IE nonexistant" },
	{ Q850_CAUSE_INVALID_IE_CONTENTS, "Invalid IE content" },
	{ Q850_CAUSE_WRONG_CALL_STATE, "Wrong call state" },
	{ Q850_CAUSE_RECOVERY_ON_TIMER_EXPIRE, "Recovery on timer expire" },
	{ Q850_CAUSE_MANDATORY_IE_LENGTH_ERROR, "Mandatory IE length error" },
	{ Q850_CAUSE_PROTOCOL_ERROR, "Protocol error" },
	{ Q850_CAUSE_INTERWORKING, "Interworking, unspecified" },

	{ 0, NULL }
};


const char *Q850CauseGetName(const int cause)
{
	struct Q850CauseName *ptr = Q850CauseNames;

	while (ptr->name) {
		if (ptr->id == cause) {
			return ptr->name;
		}
		ptr++;
	}
	return "Unknown";
}
