/*
 * T.50 - ISO/IEC 646
 *
 *	"International Reference Alphabet (IRA / IA5)"
 *
 * Collection of helper functions
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "T50.h"

int ia5_valid_string(const char *in, const int size)
{
	for (int i = 0; i < size && in[i]; i++)
		if (!isia5(in[i])) return 0;
	return 1;
}

int ia5_valid_number(const char *in, const int size)
{
	for (int i = 0; i < size && in[i]; i++)
		if (!isia5digit(in[i])) return 0;
	return 1;
}

#ifdef HAVE_ICONV
#include <iconv.h>

#define ICONV_CONVERT(_name, _to, _from)				\
	int _name(const char *in, const int isize, char *out, int *osize)	\
	{								\
		char *inp = (char *)in, *outp = out;			\
		int inleft = isize, outleft = *osize, res = 0;		\
		iconv_t cv = iconv_open(_to, _from);			\
		if (cv == -1)						\
			return -1;					\
									\
		if (iconv(cv, &inp, &inleft, &outp, &outleft) < 0) {	\
			switch (errno) {				\
			case E2BIG:					\
				res = -1;				\
				break;					\
			default:					\
				break;					\
			}						\
		}							\
		*osize -= outleft;					\
		iconv_close(cv);					\
		return res;						\
	}
#else
#define ICONV_CONVERT(_name, _to, _from)				\
	int _name(const char *in, const int isize, char *out, int *osize)	\
	{								\
		return -1;						\
	}
#endif

ICONV_CONVERT(ia5_from_utf8, "ISO_646.IRV:1991//TRANSLIT", "UTF-8")
ICONV_CONVERT(ia5_to_utf8,   "UTF-8", "ISO_646.IRV:1991")

ICONV_CONVERT(ia5_from_latin9, "ISO_646.IRV:1991//TRANSLIT", "ISO_8859-15")
ICONV_CONVERT(ia5_to_latin9,   "ISO_8859-15", "ISO_646.IRV:1991")

ICONV_CONVERT(ia5_from_latin1, "ISO_646.IRV:1991//TRANSLIT", "ISO_8859-1")
ICONV_CONVERT(ia5_to_latin1,   "ISO_8859-1", "ISO_646.IRV:1991")
