#ifndef _Q931_DIALECT_H_
#define _Q931_DIALECT_H_

/**
 * Dialect flags
 */
enum {
	Q931_DIALECT_NONE = 0,
	Q931_DIALECT_LOCKING_SHIFT_ONLY = (1 << 0),		/*!< Only locking shifts allowed */
};

/* public version of struct */
struct Q931Dialect;

/*
 *
 */
void Q931DialectRegistryInit(void);

void Q931DialectRegister(L3UCHAR id, L3INT mode, void (* create_func)(struct Q931Dialect *dialect));
struct Q931Dialect *Q931DialectGet(L3UCHAR id, L3INT mode);

/* Codesets */
void Q931DialectAddCodeset(struct Q931Dialect *dialect, L3UCHAR codeset);
L3BOOL Q931DialectIsValidCodeset(struct Q931Dialect *dialect, L3UCHAR codeset);

/* Flags */
void Q931DialectSetFlag(struct Q931Dialect *dialect, L3INT flag);
L3BOOL Q931DialectIsSetFlag(struct Q931Dialect *dialect, L3INT flag);

/* Message proc table */
void Q931DialectSetMesProc(struct Q931Dialect *dialect, L3UCHAR mes, q931proc_func_t *Q931ProcFunc, q931umes_func_t *Q931UmesFunc, q931pmes_func_t *Q931PmesFunc);

/* IE proc table */
void Q931DialectSetIEProc(struct Q931Dialect *dialect, L3UCHAR iec, q931pie_func_t *PieProc, q931uie_func_t *UieProc, q931die_func_t *DieProc);

/* Timer proc table */
void Q931DialectSetTimerProcAll(struct Q931Dialect *dialect, q931timeout_func_t *Q931TimeoutProc);
void Q931DialectSetTimerProc(struct Q931Dialect *dialect, L3UCHAR timer, q931timeout_func_t *Q931TimeoutProc);
void Q931DialectSetTimeout(struct Q931Dialect *dialect, L3UCHAR timer, q931timer_t timeout);

q931proc_func_t *Q931DialectGetMesProc(const struct Q931Dialect *dialect, L3UCHAR type);
q931proc_func_t *Q931DialectGetInvalidMesProc(const struct Q931Dialect *dialect);
q931proc_func_t *Q931DialectGetUnknownMesProc(const struct Q931Dialect *dialect);
void Q931DialectSetInvalidMesProc(struct Q931Dialect *dialect, q931proc_func_t proc);
void Q931DialectSetUnknownMesProc(struct Q931Dialect *dialect, q931proc_func_t proc);
q931umes_func_t *Q931DialectGetUmesProc(const struct Q931Dialect *dialect, L3UCHAR type);
q931pmes_func_t *Q931DialectGetPmesProc(const struct Q931Dialect *dialect, L3UCHAR type);
q931uie_func_t *Q931DialectGetUieProc(const struct Q931Dialect *dialect, L3UCHAR type);
q931pie_func_t *Q931DialectGetPieProc(const struct Q931Dialect *dialect, L3UCHAR type);
q931die_func_t *Q931DialectGetDieProc(const struct Q931Dialect *dialect, L3UCHAR type);
q931timeout_func_t *Q931DialectGetTimerProc(const struct Q931Dialect *dialect, L3UCHAR timer);
q931timer_t Q931DialectGetTimeout(const struct Q931Dialect *dialect, L3UCHAR timer);

void Q931DialectSetMesIEMap(struct Q931Dialect *dialect, struct Q931MessageIE *map);
struct Q931MessageIE *Q931DialectGetMesIEMap(struct Q931Dialect *dialect);

void Q931DialectAddStateEntry(struct Q931Dialect *dialect, L3INT state, L3INT mes, L3UCHAR flags);
L3BOOL Q931DialectIsEventLegal(struct Q931Dialect *dialect, L3INT state, L3INT mes, L3UCHAR dirflags);

L3INT Q931DialectGetID(const struct Q931Dialect *dialect);
Q931Dialect_t Q931DialectGetType(const struct Q931Dialect *dialect);
Q931NetUser_t Q931DialectGetMode(const struct Q931Dialect *dialect);

const char * Q931DialectGetName(const struct Q931Dialect *dialect);
void Q931DialectSetName(struct Q931Dialect *dialect, const char *name);

#endif
