/*
 *
 *
 *
 */
#ifndef _Q931_PRIVATE_H_
#define _Q931_PRIVATE_H_

/*
 * Private typedefs
 */
struct Q931MesgErrors;

typedef struct {
	L3INT		State;		/*!< State      */
	L3INT		Message;	/*!< Message ID */
	L3UCHAR		Flags;		/*!< MSGF_* flags, e.g. allowed source of message */
} Q931State;

typedef enum {
	Q931_MSG_FROM_L2 = 2,
	Q931_MSG_FROM_L4 = 4
} q931_msg_from_t;

typedef enum {
	Q931_MSG_INCOMING = 0,
	Q931_MSG_OUTGOING
} q931_msg_direction_t;

struct strstream;

typedef L3INT (q931proc_func_t) (Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);

typedef L3INT (q931umes_func_t) (Q931_TrunkInfo_t *trunk, L3UCHAR *IBuf, Q931mes_Generic *OBuf, L3INT IOff, L3INT Size, struct Q931MesgErrors *Errs);
typedef L3INT (q931pmes_func_t) (Q931_TrunkInfo_t *trunk, Q931mes_Generic *IBuf, L3INT ISize, L3UCHAR *OBuf, L3INT *OSize);

typedef L3INT (q931uie_func_t) (Q931_TrunkInfo_t *trunk, Q931mes_Generic *pMsg, L3UCHAR * IBuf, L3UCHAR * OBuf, L3INT *IOff, L3INT *OOff);
typedef L3INT (q931pie_func_t) (Q931_TrunkInfo_t *trunk, L3UCHAR *IBuf, L3UCHAR *OBuf, L3INT *Octet);
typedef L3INT (q931die_func_t) (Q931_TrunkInfo_t *trunk, Q931ie_Generic *ie, struct strstream *ostream);

typedef L3INT (q931timeout_func_t) (Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
typedef L3ULONG q931timer_t;


/**
 * Message <-> (mandatory) IE table
 */
enum {
	Q931_IEF_NONE        = 0,		/*!< None */
	Q931_IEF_MANDATORY   = (1 << 0),	/*!< IE is mandatory */
	/* Direction of IE */
	Q931_IEF_TO_NET      = (1 << 14),	/*!< User -> Network */
	Q931_IEF_TO_USER     = (1 << 15),	/*!< Network -> User */
};

enum {
	/*
	 * Codesets
	 */
	Q931_IE_CODESET_0   = (1 << 0),		/*!< Codeset 0 */
	Q931_IE_CODESET_1   = (1 << 1),		/*!< Codeset 1 */
	Q931_IE_CODESET_2   = (1 << 2),		/*!< Codeset 2 */
	Q931_IE_CODESET_3   = (1 << 3),		/*!< Codeset 3 */
	Q931_IE_CODESET_4   = (1 << 4),		/*!< Codeset 4 */
	Q931_IE_CODESET_5   = (1 << 5),		/*!< Codeset 5 */
	Q931_IE_CODESET_6   = (1 << 6),		/*!< Codeset 6 */
	Q931_IE_CODESET_7   = (1 << 7),		/*!< Codeset 7 */
};

#define Q931_IEF_TO_BOTH    (Q931_IEF_TO_NET | Q931_IEF_TO_USER)
#define Q931_IE_CODESET_ALL (Q931_IE_CODESET_0 | Q931_IE_CODESET_1 | Q931_IE_CODESET_2 | Q931_IE_CODESET_3 | Q931_IE_CODESET_4 | Q931_IE_CODESET_5 | Q931_IE_CODESET_6 | Q931_IE_CODESET_7)

/*
 * TODO: better names...
 */
struct Q931MessageIEEntry {
	const L3UCHAR id;	/*!< IE ID */
	const L3UCHAR minsize;	/*!< Min. IE size */
	const L3UCHAR maxsize;	/*!< Max. IE size (or 0 for fixed size IE) */
	const L3UCHAR codeset;	/*!< Codeset (Q931_IE_CODESET_*) */
	const L3INT   flags;	/*!< Flags for this IE (Q931_IEF_*) */
};

struct Q931MessageIE {
	const L3UCHAR id;				/*!< Message ID  */
	const L3UCHAR protdisc;				/*!< Protocol discriminator */
	const L3CHAR *name;				/*!< Name */
	const struct Q931MessageIEEntry ies[Q931MAXIE];	/*!< List of IEs */
};


/**
 * Message direction and other flags
 *
 * \note Private!
 */
enum {
	MSGF_FROM_L2 = 2,
	MSGF_FROM_L4 = 4,
	MSGF_FROM_BOTH = 6,
};

/**
 * Q931 Trunk feature flags
 * \note: private
 */
enum {
	Q931_TFLAG_STATUS_ENQUIRY   = (1 <<  0),	/*!< STATUS ENQUIRY on message error */

	Q931_TFLAG_AUTO_SETUP_ACK   = (1 << 16),	/*!< */
	Q931_TFLAG_AUTO_SERVICE_ACK = (1 << 17),	/*!< */
	Q931_TFLAG_AUTO_CONNECT_ACK = (1 << 18),	/*!< */
	Q931_TFLAG_AUTO_RESTART_ACK = (1 << 19),	/*!< */

	/*
	 * Message check relaxation flags (use with care...)
	 */
	Q931_TFLAG_IGNORE_UNKNOWN_MSG = (1 << 28),	/*!< Ignore unknown messages */

	/*
	 * IE check relaxation flags (use with care...)
	 */
	Q931_TFLAG_IGNORE_UNKNOWN_IE = (1 << 29),	/*!< Ignore unknown IEs (= that we do not have a handler for) */
	Q931_TFLAG_IGNORE_ILLEGAL_IE = (1 << 30),	/*!< Ignore illegal IEs (= that isn't valid for this message, or simply not in our list of allowed ones) */
};

void Q931TrunkSetFlag(Q931_TrunkInfo_t *trunk, const L3INT flag);
void Q931TrunkClearFlag(Q931_TrunkInfo_t *trunk, L3INT flag);
void Q931TrunkClearAllFlags(Q931_TrunkInfo_t *trunk);
L3BOOL Q931TrunkIsSetFlag(const Q931_TrunkInfo_t *trunk, const L3INT flag);

/*
 *
 */
L3ULONG Q931GetTime(void);

L3INT Q931Log(const Q931_TrunkInfo_t *trunk, Q931LogLevel_t level, const char *fmt, ...);
L3INT Q931LogRaw(const Q931_TrunkInfo_t *trunk, Q931LogLevel_t level, const char *str);

L3INT Q931GetUniqueCRV(Q931_TrunkInfo_t *trunk);

L3INT Q931Tx34(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, int size);
L3INT Q931Tx32(Q931_TrunkInfo_t *trunk, L3UCHAR bcast, Q931mes_Generic *msg, int size);

L3INT Q931ProcUnknownMessage(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcUnexpectedMessage(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);

L3INT Q931ProcDummy(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931UmesDummy(Q931_TrunkInfo_t *trunk, L3UCHAR *IBuf, Q931mes_Generic *OBuf, L3INT IOff, L3INT Size, struct Q931MesgErrors *Errs);
L3INT Q931PmesDummy(Q931_TrunkInfo_t *trunk, Q931mes_Generic *IBuf, L3INT ISize, L3UCHAR *OBuf, L3INT *OSize);
L3INT Q931UieDummy(Q931_TrunkInfo_t *trunk, Q931mes_Generic *pMsg, L3UCHAR * IBuf, L3UCHAR * OBuf, L3INT *IOff, L3INT *OOff);
L3INT Q931PieDummy(Q931_TrunkInfo_t *trunk, L3UCHAR *IBuf, L3UCHAR *OBuf, L3INT *Octet);
L3INT Q931TxDummy(Q931_TrunkInfo_t *trunk, L3UCHAR * b, L3INT n);
L3INT Q931TimeoutDummy(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);

L3INT Q931MesgHeader(Q931_TrunkInfo_t *trunk, Q931mes_Generic *mes, L3UCHAR *OBuf, L3INT Size, L3INT *IOff);

L3INT Q931StatusEnquiryResponse(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, L3UCHAR causeval);

L3INT Q931Umes_Generic(Q931_TrunkInfo_t *trunk, L3UCHAR *IBuf, Q931mes_Generic *mes, L3INT IOff, L3INT Size, struct Q931MesgErrors *Errs);
L3INT Q931Pmes_Generic(Q931_TrunkInfo_t *trunk, Q931mes_Generic *IBuf, L3INT ISize, L3UCHAR *OBuf, L3INT *OSize);
L3INT Q931Dmes_Generic(Q931_TrunkInfo_t *trunk, Q931mes_Generic *msg, q931_msg_direction_t direction);


/**
 * Message parser error codes
 * @note: Maximum value is 255 (id field in Q931MesgErrors.ie_errors is 8 bits!)
 */
enum {
	/* Order must match that of Q931MesgErrorNames in Q931mes.c! */
	Q931_MSGE_NONE = 0,
	Q931_MSGE_UNKNOWN,
	Q931_MSGE_OVERFLOW,
	Q931_MSGE_NOT_ALLOWED,
	Q931_MSGE_INVALID_DIRECTION,
	Q931_MSGE_INVALID_SIZE,
	Q931_MSGE_INVALID_CODESET,
	Q931_MSGE_MANDATORY_MISSING,
	Q931_MSGE_COUNT
};

#define Q931_MSGE_ERR_MAX	10

/**
 * Message parser error reporting
 */
struct Q931MesgErrors {
	char    id;			/*!< Message id                */
	int	nr_errors;		/*!< Number of IE errors       */
	int	nr_overflow;		/*!< Count additional errors   */
	struct {
		char	id;		/*!< IE id                     */
		char	size;		/*!< IE size                   */
		char	error;		/*!< Error code                */
		char	offset;		/*!< Offset of IE in message   */
	} ie_errors[Q931_MSGE_ERR_MAX];	/*!< Per IE reporting of error */
};

L3INT Q931MesgErrorsInit(struct Q931MesgErrors *merrs, const char id);
L3INT Q931MesgErrorsAddIE(struct Q931MesgErrors *merrs, const char id, const char size, const int offset, const int error);
L3INT Q931MesgErrorsCount(struct Q931MesgErrors *merrs);
L3INT Q931MesgErrorsPrint(Q931_TrunkInfo_t *trunk, const struct Q931MesgErrors *merrs, const unsigned char *IBuf, const int Size);

/*
 * High-level
 */
L3INT Q931Proc(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcInvalid(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931Umes(Q931_TrunkInfo_t *trunk, L3UCHAR id, L3UCHAR *IBuf, Q931mes_Generic *OBuf, L3INT IOff, L3INT Size, struct Q931MesgErrors *Errs);
L3INT Q931Pmes(Q931_TrunkInfo_t *trunk, L3UCHAR id, Q931mes_Generic *IBuf, L3INT ISize, L3UCHAR *OBuf, L3INT *OSize);
L3INT Q931Uie(Q931_TrunkInfo_t *trunk, L3UCHAR id, Q931mes_Generic *pMes, L3UCHAR *IBuf, L3UCHAR *OBuf, L3INT *IOff, L3INT *OOff);
L3INT Q931Pie(Q931_TrunkInfo_t *trunk, L3UCHAR id, L3UCHAR *IBuf, L3UCHAR *OBuf, L3INT *Octet);
L3INT Q931Die(Q931_TrunkInfo_t *trunk, Q931ie_Generic *ie, struct strstream *ostream);
L3INT Q931Timeout(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, L3UCHAR timer);

L3BOOL Q931UieIsNull(Q931_TrunkInfo_t *trunk, L3UCHAR id);
L3BOOL Q931PieIsNull(Q931_TrunkInfo_t *trunk, L3UCHAR id);
L3BOOL Q931DieIsNull(Q931_TrunkInfo_t *trunk, L3UCHAR id);

#include "Q931dialect.h"

/*
 * Q.931 base dialect
 */
void Q931CreateTE(struct Q931Dialect *d);
void Q931CreateNT(struct Q931Dialect *d);

/*
 * TE-mode message handling
 */
L3INT Q931ProcAlertingTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcCallProceedingTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcConnectTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcConnectAckTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcProgressTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSetupTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSetupAckTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcResumeTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcResumeAckTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcResumeRejectTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSuspendTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSuspendAckTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSuspendRejectTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcUserInformationTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcDisconnectTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcReleaseTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcReleaseCompleteTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcRestartTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcRestartAckTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcCongestionControlTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcInformationTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcNotifyTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcStatusTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcStatusEnquiryTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSegmentTE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);

L3INT Q931ProcTimeoutT301TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT302TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT303TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT304TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT305TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT308TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT309TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT310TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT311TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT313TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT314TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT316TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT317TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT318TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT319TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT321TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT322TE(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);

/*
 * NT-mode message handling
 */
L3INT Q931ProcAlertingNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcCallProceedingNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcConnectNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcConnectAckNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcProgressNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSetupNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSetupAckNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcResumeNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcResumeAckNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcResumeRejectNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSuspendNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSuspendAckNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSuspendRejectNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcUserInformationNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcDisconnectNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcReleaseNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcReleaseCompleteNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcRestartNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcRestartAckNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcCongestionControlNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcInformationNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcNotifyNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcStatusNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcStatusEnquiryNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);
L3INT Q931ProcSegmentNT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call, Q931mes_Generic *msg, q931_msg_from_t from);

L3INT Q931ProcTimeoutT301NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT302NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT303NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT304NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT305NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT306NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT307NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT308NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT309NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT310NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT312NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT313NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT314NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT316NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT317NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT320NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT321NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);
L3INT Q931ProcTimeoutT322NT(Q931_TrunkInfo_t *trunk, struct Q931_Call *call);

/*****************************************************************************

  Q.931 Information Element Pack/Unpack functions. Implemented in Q931ie.c

*****************************************************************************/
L3INT Q931ReadExt(L3UCHAR * IBuf, L3INT Off);
L3USHORT Q931Uie_CRV(Q931_TrunkInfo_t *pTrunk,L3UCHAR * IBuf, L3UCHAR *OBuf, L3INT *IOff, L3INT *OOff);

L3INT Q931Uie_Generic(Q931_TrunkInfo_t *pTrunk, Q931mes_Generic *pMsg, L3UCHAR *IBuf, L3UCHAR *OBuf, L3INT *IOff, L3INT *OOff);
L3INT Q931Pie_Generic(Q931_TrunkInfo_t *pTrunk, L3UCHAR *IBuf, L3UCHAR *OBuf, L3INT *Octet);
L3INT Q931Die_Generic(Q931_TrunkInfo_t *pTrunk, Q931ie_Generic *ie, struct strstream *ostream);

q931pie_func_t Q931Pie_ChangeStatus;
q931pie_func_t Q931Pie_BearerCap;
q931pie_func_t Q931Pie_ChanID;
q931pie_func_t Q931Pie_ProgInd;
q931pie_func_t Q931Pie_Display;
q931pie_func_t Q931Pie_Signal;
q931pie_func_t Q931Pie_HLComp;
q931pie_func_t Q931Pie_Segment;
q931pie_func_t Q931Pie_DateTime;
q931pie_func_t Q931Pie_Cause;
q931pie_func_t Q931Pie_SendComplete;
q931pie_func_t Q931Pie_KeypadFac;
q931pie_func_t Q931Pie_NotifInd;
q931pie_func_t Q931Pie_CallID;
q931pie_func_t Q931Pie_RepeatInd;
q931pie_func_t Q931Pie_NetFac;
q931pie_func_t Q931Pie_CallingNum;
q931pie_func_t Q931Pie_CallingSub;
q931pie_func_t Q931Pie_CalledNum;
q931pie_func_t Q931Pie_CalledSub;
q931pie_func_t Q931Pie_CalledNum;
q931pie_func_t Q931Pie_TransNetSel;
q931pie_func_t Q931Pie_LLComp;
q931pie_func_t Q931Pie_CallState;
q931pie_func_t Q931Pie_RestartInd;
q931pie_func_t Q931Pie_UserUser;
q931pie_func_t Q931Pie_GenericDigits;
q931pie_func_t Q931Pie_CongLevel;
q931pie_func_t Q931Pie_RevChargeInd;

q931uie_func_t Q931Uie_ChangeStatus;
q931uie_func_t Q931Uie_BearerCap;
q931uie_func_t Q931Uie_ChanID;
q931uie_func_t Q931Uie_ProgInd;
q931uie_func_t Q931Uie_Display;
q931uie_func_t Q931Uie_Signal;
q931uie_func_t Q931Uie_HLComp;
q931uie_func_t Q931Uie_Segment;
q931uie_func_t Q931Uie_DateTime;
q931uie_func_t Q931Uie_Cause;
q931uie_func_t Q931Uie_SendComplete;
q931uie_func_t Q931Uie_KeypadFac;
q931uie_func_t Q931Uie_NotifInd;
q931uie_func_t Q931Uie_CallID;
q931uie_func_t Q931Uie_RepeatInd;
q931uie_func_t Q931Uie_NetFac;
q931uie_func_t Q931Uie_CallingNum;
q931uie_func_t Q931Uie_CallingSub;
q931uie_func_t Q931Uie_CalledNum;
q931uie_func_t Q931Uie_CalledSub;
q931uie_func_t Q931Uie_TransNetSel;
q931uie_func_t Q931Uie_LLComp;
q931uie_func_t Q931Uie_CallState;
q931uie_func_t Q931Uie_RestartInd;
q931uie_func_t Q931Uie_UserUser;
q931uie_func_t Q931Uie_GenericDigits;
q931uie_func_t Q931Uie_CongLevel;
q931uie_func_t Q931Uie_RevChargeInd;

q931die_func_t Q931Die_ChangeStatus;
q931die_func_t Q931Die_BearerCap;
q931die_func_t Q931Die_ChanID;
q931die_func_t Q931Die_ProgInd;
q931die_func_t Q931Die_Display;
q931die_func_t Q931Die_Signal;
q931die_func_t Q931Die_HLComp;
q931die_func_t Q931Die_Segment;
q931die_func_t Q931Die_DateTime;
q931die_func_t Q931Die_Cause;
q931die_func_t Q931Die_SendComplete;
q931die_func_t Q931Die_KeypadFac;
q931die_func_t Q931Die_NotifInd;
q931die_func_t Q931Die_CallID;
q931die_func_t Q931Die_RepeatInd;
q931die_func_t Q931Die_NetFac;
q931die_func_t Q931Die_CallingNum;
q931die_func_t Q931Die_CallingSub;
q931die_func_t Q931Die_CalledNum;
q931die_func_t Q931Die_CalledSub;
q931die_func_t Q931Die_TransNetSel;
q931die_func_t Q931Die_LLComp;
q931die_func_t Q931Die_CallState;
q931die_func_t Q931Die_RestartInd;
q931die_func_t Q931Die_UserUser;
q931die_func_t Q931Die_GenericDigits;
q931die_func_t Q931Die_CongLevel;
q931die_func_t Q931Die_RevChargeInd;

#endif /* _Q931_PRIVATE_H_ */
