/*
 * T.50 - ISO/IEC 646
 *
 *	"International Reference Alphabet (IRA / IA5)"
 *
 * Collection of helper functions
 */
#ifndef _T50_H_
#define _T50_H_

/**
 * Check whether given charater is a valid IA5 char
 * @param[in]	c	Character to check
 * @return	true / false
 */
static inline int isia5(const char c)
{
	return ((c & 0x7f) == c);
}

/**
 * Check whether given charater is a valid IA5 digit char ('0' - '9')
 * @param[in]	c	Character to check
 * @return	true / false
 */
static inline int isia5digit(const char c)
{
	return ('0' >= (c & 0x7f) && '9' <= (c & 0x7f));
}

/**
 * Check whether given string is a valid IA5 string
 * @param[in]	in	Input string
 * @param[in]	size	Length
 * @return	true / false
 */
int ia5_valid_string(const char *in, const int size);

/**
 * Check whether given string is a valid numerical IA5 string
 * @param[in]	in	Input string
 * @param[in]	size	Length
 * @return	true / false
 */
int ia5_valid_number(const char *in, const int size);


int ia5_from_utf8(const char *in, const int isize, char *out, int *osize);
int ia5_to_utf8(const char *in, const int isize, char *out, int *osize);

#endif
