/*
 * X.213 NSAP address handling (for Q.931 Calling/-ed Party Subaddress IE)
 *
 *
 */
#ifndef _X213_H_
#define _X213_H_

#if defined(PLATFORM_WINDOWS)
#include <winsock2.h>
#include <ws2tcpip.h>
#if defined(PLATFORM_WIN64)
#include <inaddr.h>
#include <in6addr.h>
#endif /* PLATFORM_WIN64 */
#else
#include <arpa/inet.h>
#include <netinet/in.h>	/* in_addr, in6_addr */
#include <sys/socket.h>	/* AF_INET, AF_INET6 */
#endif

typedef enum {
	NSAP_IDI_NONE = 0,
	NSAP_IDI_X121,
	NSAP_IDI_ISO_DCC,
	NSAP_IDI_F69,
	NSAP_IDI_E163,
	NSAP_IDI_E164,
	NSAP_IDI_ISO_6523_ICD,
	NSAP_IDI_IANA_ICP,
	NSAP_IDI_ITU_T_IND,
	NSAP_IDI_LOCAL
} nsap_idi_format_t;

typedef enum {
	NSAP_DSP_NONE = 0,
	NSAP_DSP_DECIMAL,
	NSAP_DSP_BINARY,
	NSAP_DSP_ISO_IEC_646,
	NSAP_DSP_NATIONAL
} nsap_dsp_syntax_t;

enum {
	NSAPE_NO_ERROR       =  0,
	NSAPE_GENERR         = -1,
	NSAPE_TOO_SHORT      = -2,
	NSAPE_INVALID_FORMAT = -3,
	NSAPE_UNSUPPORTED_FORMAT = -4,
};

struct nsap_addr_f69 {
	char telex[8];
	char value[30];
};

struct nsap_addr_iso_dcc {
	unsigned short dcc;
	const char *name;
	const char *code;
	char value[35];
};

struct nsap_addr_iana_icp {
	int family;
	union {
		struct in_addr  ip4;
		struct in6_addr ip6;
	} icp_addr;
#define icp_a_ip4 icp_addr.ip4
#define icp_a_ip6 icp_addr.ip6
};

struct nsap_addr_iso_6523_icd {
	unsigned int icd;
	char value[34];
};

struct nsap_addr_x121 {
	char dnn[14];
	char value[24];
};

struct nsap_addr_e163 {
	char number[12];
	char value[26];
};

struct nsap_addr_e164 {
	char number[15];
	char value[23];
};

struct nsap_addr_itu_t_ind {
	unsigned int ind;
	char value[32];
};

struct nsap_addr_local {
	char value[38];
};

/**
 * decoded nsap address
 */
struct nsap_addr {
	nsap_idi_format_t idi;
	nsap_dsp_syntax_t dsp;
	int max_dsp_length;
	union {
		struct nsap_addr_x121          x121;
		struct nsap_addr_iso_dcc       iso_dcc;
		struct nsap_addr_f69           f69;
		struct nsap_addr_e163          e163;
		struct nsap_addr_e164          e164;
		struct nsap_addr_iso_6523_icd  iso_6523_icd;
		struct nsap_addr_iana_icp      iana_icp;
		struct nsap_addr_itu_t_ind     itu_t_ind;
		struct nsap_addr_local         local;
	} ns_addr;
#define nsap_a_x121         ns_addr.x121
#define nsap_a_iso_dcc      ns_addr.iso_dcc
#define nsap_a_f69          ns_addr.f69
#define nsap_a_e163         ns_addr.e163
#define nsap_a_e164         ns_addr.e164
#define nsap_a_iso_6523_icd ns_addr.iso_6523_icd
#define nsap_a_iana_icp     ns_addr.iana_icp
#define nsap_a_itu_t_ind    ns_addr.itu_t_ind
#define nsap_a_local        ns_addr.local
};

int nsap_afi_valid(const char afi);
int nsap_decode(struct nsap_addr *addr, char *buf, int size);
int nsap_print(struct nsap_addr *addr, char *buf, int size);

#endif
