/*
 *
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>

#include "common.h"
#include "dlist.h"


void dlist_init_head(struct dlist_head *h)
{
	h->head = h;
	h->next = h;
	h->prev = h;
}

void dlist_init(struct dlist_head *e)
{
	e->head = NULL;
	e->next = NULL;
	e->prev = NULL;
}

void dlist_insert_after(struct dlist_head *elem, struct dlist_head *enew)
{
	enew->head = elem->head;
	enew->prev = elem;

	if (elem->next == elem->head) {
		enew->next = enew->head;
		enew->head->prev = enew;	// last element in list, update pointer in head
	} else {
		elem->next->prev = enew;
		enew->next = elem->next;
	}

	elem->next = enew;
}


void dlist_insert_before(struct dlist_head *elem, struct dlist_head *enew)
{
	/*
	 * Cannot insert before list head
	 */
	if (elem->head == elem)
		return;

	enew->head = elem->head;
	enew->prev = elem->prev;
	elem->prev->next = enew;
	elem->prev = enew;
	enew->next = elem;
}

void dlist_insert_tail(struct dlist_head *list, struct dlist_head *enew)
{
	if (list->head->prev == NULL) {
		/* empty list */
		enew->head = list->head;
		enew->prev = list->head;
		enew->next = list->head;
		list->head->prev = enew;
		list->head->next = enew;
		return;
	}

	if (list->head->prev == list->head)
		dlist_insert_after(list->head, enew);
	else
		dlist_insert_after(list->head->prev, enew);
}

void dlist_remove(struct dlist_head *elem)
{
	/*
	 * Cannot remove list head
	 */
	if (elem->head == elem)
		return;

	if (elem->next != elem->head) {
		elem->prev->next = elem->next;
		elem->next->prev = elem->prev;
	} else {
		elem->prev->next = elem->head;
		elem->head->prev = elem->prev;	// was last element in list, update head pointer
	}
}

struct dlist_head *dlist_first(const struct dlist_head *list)
{
	if (list->head && list->head->next != list->head)
		return list->head->next;

	return NULL;
}

struct dlist_head *dlist_last(const struct dlist_head *list)
{
	if (list->prev && list->head->prev != list->head)
		return list->head->prev;

	return NULL;
}

struct dlist_head *dlist_next(const struct dlist_head *elem)
{
	if (elem->next != elem->head)
		return elem->next;

	return NULL;
}

struct dlist_head *dlist_prev(const struct dlist_head *elem)
{
	if (elem->prev && elem->head != elem)
		return elem->prev;

	return NULL;
}

struct dlist_head *dlist_pop_head(const struct dlist_head *elem)
{
	struct dlist_head *h = dlist_first(elem);
	if (h) dlist_remove(h);
	return h;
}

struct dlist_head *dlist_pop_tail(const struct dlist_head *elem)
{
	struct dlist_head *h = dlist_last(elem);
	if (h) dlist_remove(h);
	return h;
}

int dlist_is_head(const struct dlist_head *elem)
{
	return (elem->head == elem);
}


int dlist_is_first(const struct dlist_head *elem)
{
	return (elem->prev == elem->head);
}


int dlist_is_last(const struct dlist_head *elem)
{
	return (elem->next == elem->head);
}

void dlist_replace(struct dlist_head *old, struct dlist_head *enew)
{
	enew->next = old->next;
	if (old->next != old->head)
		enew->next->prev = enew;

	enew->prev = old->prev;
	enew->prev->next = enew;

	enew->head = old->head;
}
