/*
 *
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>

#include "msgb.h"
#include "msgb_fifo.h"

/*******************************************************************************
 *
 *******************************************************************************/

int msgb_fifo_init(struct msgb_fifo *fifo, const int size)
{
	if (!fifo || size <= 1)
		return -1;

	memset(fifo, 0, sizeof(*fifo));
	dlist_init_head(&fifo->entries);
	fifo->size = size;
	return 0;
}

struct msgb_fifo *msgb_fifo_alloc(const int size)
{
	struct msgb_fifo *fifo = NULL;

	if (size <= 1)
		return NULL;

	fifo = malloc(sizeof(*fifo));
	if (!fifo)
		return NULL;

	if (msgb_fifo_init(fifo, size) < 0) {
		free(fifo);
		return NULL;
	}

	return fifo;
}

int msgb_fifo_destroy(struct msgb_fifo *fifo)
{
	if (!fifo)
		return -1;
	free(fifo);
	return 0;
}

/*******************************************************************************
 *
 *******************************************************************************/

int msgb_fifo_size(const struct msgb_fifo *fifo)
{
	return (fifo) ? fifo->size : -1;
}

int msgb_fifo_length(const struct msgb_fifo *fifo)
{
	return (fifo) ? fifo->nr_entries : -1;
}

int msgb_fifo_empty(const struct msgb_fifo *fifo)
{
	return !!(!fifo || !fifo->nr_entries);
}

int msgb_fifo_full(const struct msgb_fifo *fifo)
{
	return !!(!fifo || fifo->nr_entries == fifo->size);
}


/*******************************************************************************
 *
 *******************************************************************************/

int msgb_fifo_enqueue(struct msgb_fifo *fifo, struct msgb *msg)
{
	if (!fifo || !msg)
		return -1;
	if (fifo->nr_entries == fifo->size)
		return -1;

	dlist_insert_tail(&fifo->entries, &msg->list);
	fifo->nr_entries++;
	return 0;
}

struct msgb *msgb_fifo_dequeue(struct msgb_fifo *fifo)
{
	struct dlist_head *entry = NULL;

	if (!fifo || !fifo->nr_entries)
		return NULL;
	if ((entry = dlist_pop_head(&fifo->entries)) == NULL)
		return NULL;

	fifo->nr_entries--;
	return dlist_entry(entry, struct msgb, list);
}
