/*
 *
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "strstream.h"

int strstream_init(struct strstream *s, const int size)
{
	if (!s || !size)
		return -1;

	s->buf = malloc(size);
	if (!s->buf)
		return -1;

	memset(s->buf, 0, size);
	s->offset = 0;
	s->size   = size;
	return 0;
}

int strstream_init_static(struct strstream *s, char *buf, const int size)
{
	if (!s || !size || !buf)
		return -1;

	s->buf = buf;
	memset(s->buf, 0, size);
	s->offset = 0;
	s->size   = size;
	return 0;
}

int strstream_printf(struct strstream *s, const char *fmt, ...)
{
	int ret = 0;
	va_list ap;

	if (!s || !fmt)
		return -1;

	va_start(ap, fmt);
	ret = vsnprintf(&s->buf[s->offset], s->size - s->offset, fmt, ap);
	va_end(ap);

	if (ret > 0) {
		s->offset += ret;
		s->buf[s->size - 1] = '\0';
	}
	return ret;
}

int strstream_puts(struct strstream *s, const char *str)
{
	if (!s || !str)
		return -1;

	if (strstream_left(s) <= strlen(str))
		return -1;

	strncat(&s->buf[s->offset], str, s->size - s->offset);
	s->buf[s->size - 1] = '\0';
	s->offset = strlen(s->buf);
	return 0;
}

int strstream_printhex(struct strstream *s, const char *buf, const int size)
{
	static const char hex[16] = {
		'0', '1', '2', '3', '4', '5',
		'6', '7', '8', '9', 'a', 'b', 'c',
		'e', 'f'
	};

	if (!s || !buf || !size)
		return -1;

	if (strstream_left(s) < ((size << 1) + size))
		return -1;

	for (int x = 0; x < size; x++) {
		s->buf[s->offset++] = hex[((buf)[x] & 0xf0) >> 4];
		s->buf[s->offset++] = hex[((buf)[x] & 0x0f)];
		if (x + 1 < size)
			s->buf[s->offset++] = ' ';
	}
	s->buf[s->offset] = '\0';

	return 0;
}

int strstream_length(struct strstream *s)
{
	return s->offset;
}

int strstream_size(struct strstream *s)
{
	return s->size;
}

int strstream_left(struct strstream *s)
{
	return (s->size - s->offset);
}

const char *strstream_get(struct strstream *s)
{
	return s->buf;
}
